## GOAL: 
## 1. implement different disease mapping models using
## - no covariate
## - a linear covariate
## - a smooth covariate
## - space-varying regression
## 2. model choice based on
## - DIC
## - CPO

library(INLA)
library(spam)

data(Germany)
g = system.file("demodata/germany.graph", package="INLA")
#source(system.file("demodata/Bym-map.R", package="INLA"))

head(Germany)

## standard BYM model 

# we need to add a new column as an index cannot be
# used twice, i.e the index "region" cannot be used 
# for two f-function, the structured and unstructured
# random effect.
Germany = cbind(Germany, region.unstruc= Germany$region)

formula1 = Y ~ f(region, model="besag", graph=g) + 
			   f(region.unstruc, model="iid")

model1 = inla(formula1, family="poisson", E=E, data=Germany,
	control.compute=list(dic=TRUE, cpo=TRUE))

table(model1$cpo$failure)

# recompute those CPO values which are not accurate
model1 = inla.cpo(model1)

## BYM model with linear covariate
formula2 = Y ~ f(region, model="besag", graph=g) + 
			   f(region.unstruc, model="iid") + x

model2 = inla(formula2, family="poisson", E=E, data=Germany,
	control.compute=list(dic=TRUE, cpo=TRUE))

model2 = inla.cpo(model2)

## BYM model with smooth covariate
formula3 = Y ~ f(region, model="besag", graph=g) + 
			   f(region.unstruc, model="iid") +
	           f(x, model="rw2")

model3 = inla(formula3, family="poisson", E=E, data=Germany,
	control.compute=list(dic=TRUE, cpo=TRUE))

model3 = inla.cpo(model3)

matplot(model3$summary.random$x[,c("0.025quant", "0.5quant", "0.975quant")], 
	col=1, type="l", lty=c(2,1,2))

## BYM models with space-varying regression

Germany = cbind(Germany, region.x= Germany$region)

formula4 = Y ~ f(region, model="besag", graph=g) + 
			   f(region.unstruc, model="iid") +
	           f(region.x, x, model="besag", graph=g, constr=FALSE)

model4 = inla(formula4, family="poisson", E=E, data=Germany,
	control.compute=list(dic=TRUE, cpo=TRUE))

model4 = inla.cpo(model4)

## compare models using the deviance information criterion (DIC)
c(model1$dic$dic, model2$dic$dic, model3$dic$dic, model4$dic$dic)
# The model with linear covariate effect seems 
# to be the best according to DIC

## compare models using the log-score
lS = function(inlaObj){
	return(-sum(log(inlaObj$cpo$cpo)))
}

c(lS(model1), lS(model2), lS(model3), lS(model4))
# The model with linear covariate effect seems 
# to be the best according to the log-score

## If you are interested in the marginal of a latent
## effect on exp-scale you can transform the whole marginal.
## For example:

transMarg = inla.tmarginal(function(x) exp(x), model1$marginals.fixed[[1]])

## Plotting
par(mfrow=c(2,2))
# Bym.map(model1$summary.random$region$mean)
# Bym.map(model2$summary.random$region$mean)
# Bym.map(model3$summary.random$region$mean)

# use the function provided in the spam package
library(spam)
library(fields, warn.conflict=FALSE)
library(colorspace)
col <- diverge_hcl(8)
map.landkreis(exp(model1$summary.random$region$"0.5quant"),col=col)
map.landkreis(exp(model2$summary.random$region$"0.5quant"),col=col)
map.landkreis(exp(model3$summary.random$region$"0.5quant"),col=col)
map.landkreis(exp(model4$summary.random$region$"0.5quant"),col=col)
