 ## Here we illustrate three different ways to define hyperpriors in INLA
 ## - using built-in functions
 ## - using expression
 ## - using a table with x and y values
 
 
 ## the loggamma-prior for the log precision parameter 
 ## (i.e. the internal parameterisation of the precision)
 prior.function = function(log_precision) {
     a = 1;
     b = 0.1;
     precision = exp(log_precision);
     logdens = log(b^a) - lgamma(a) + (a-1)*log_precision - b*precision;
     log_jacobian = log_precision;
     return(logdens + log_jacobian)
 }
 
 ## implementing the loggamma-prior using "expression:"
 prior.expression = "expression:
             a = 1;
             b = 0.1;
             precision = exp(log_precision);
             logdens = log(b^a) - lgamma(a)
                       + (a-1)*log_precision - b*precision;
             log_jacobian = log_precision;
             return(logdens + log_jacobian);"
             
 ## use suitable support points x
 lprec = seq(-10, 10, len=100)
 ## link the x and corresponding y values into a string which begins with "table:""
 prior.table = INLA:::inla.paste(c("table:",cbind(lprec,prior.function(lprec))))
 # simulate some data
 n = 50
 y = rnorm(n)
 
 ## 1. use the built-in loggamma prior
 r1 = inla(y~1,data = data.frame(y),
         control.family = list(hyper =
             list(prec = list(prior = "loggamma", param = c(1, 0.1)))))
 ## 2. use the definition using expression
 r2 = inla(y~1,
         data = data.frame(y),
         control.family = list(hyper =
             list(prec = list(prior = prior.expression))))
 ## 3. use a table of x and y values representing the loggamma prior
 r3 = inla(y~1,
         data = data.frame(y),
         verbose=T,
         control.family = list(hyper =
             list(prec = list(prior = prior.table))))
 ## and we verify that we get the same result using the log-marginal likelihood
 c(r1$mlik[1], r2$mlik[1], r3$mlik[1])
